Imports System.IO

Module Module1

    Sub Main()

        'purpose: provide example code of asynch. file I/O

        'steps:   1. start asynch. read and processing of 3 files of 
        '            varying lengths
        '         2. wait for callback and display to screen

        'local scope
        Dim myFiles(3) As String
        Dim i As Int16

        myFiles(0) = "c:\file1.txt"
        myFiles(1) = "c:\file2.txt"
        myFiles(2) = "c:\file3.txt"

        'call each asynch. read
        For i = 0 To 2
            Console.WriteLine("Starting file read: " & myFiles(i))
            Call asynchRead(filePath:=myFiles(i))
        Next

        'NOTE: now that file reads have started, our application can 
        '      continue processing other information and await a callback 
        '      from the read operation indicating read is complete

        'wait for the user to stop the console application
        Console.WriteLine("******** Enter 's' to stop the application.")

        'loop until users presses s key
        Do While Console.ReadLine <> "s" : Loop

    End Sub

    Private Sub asynchRead(ByVal filePath As String)

        'purpose: execute an asynch. read against a give file
        '         throw an exception if the file is not found

        'local scope
        Dim fileStream As FileStream
        Dim state As StateObject
        Dim fileInfo As FileInfo

        'check to see if the file exists
        If Not File.Exists(path:=filePath) Then

            'file does not exist = throw an exception
            Throw New Exception(message:="File not found.")

        End If

        'file exists = create an open instance of the file
        fileStream = New FileStream(path:=filePath, mode:=FileMode.Open)

        'determine size of the file to set the number of bytes
        fileInfo = New FileInfo(fileName:=filePath)
        Console.WriteLine("File length: " & fileInfo.Length)

        'crate a state object 
        state = New StateObject(filePath:=filePath, _
            byteArraySize:=fileInfo.Length)

        'set fileStream prop (useful for callback)
        state.FileStream = fileStream

        'begin the file read
        fileStream.BeginRead(array:=state.ByteArray, offset:=0, _
            numBytes:=fileInfo.Length, _
            userCallback:=New AsyncCallback(AddressOf fileRead), _
            stateObject:=state)

    End Sub

    Private Sub fileRead(ByVal asyncResult As IAsyncResult)

        'purpose: provide a callback method for asynch reads

        'local scope
        Dim state As StateObject
        Dim bytesRead As Integer

        'set the state object = to the one returned by the asynch results
        state = asyncResult.AsyncState

        'write out the path of the object read
        Console.WriteLine(state.FilePath)

        'indicate that the file was read asynch.
        If asyncResult.CompletedSynchronously Then
            Console.WriteLine("File was read synchronously.")
        Else
            Console.WriteLine("File was read asynchronously.")
        End If

        'detrmine the number of bytes read by calling EndRead
        bytesRead = state.FileStream.EndRead(asyncResult)

        'write out bytes read
        Console.WriteLine("Bytes read: " & bytesRead)

        'close the file stream
        state.FileStream.Close()

    End Sub

End Module

Public Class StateObject

    'purpose: maintain state information across asynch calls

    'class-level scope
    Private localFilePath As String
    Private localByteArray() As Byte

    'public properties
    Public FileStream As FileStream

    Public Property ByteArray() As Byte()

        'purpose: get and set ByteArray property

        Get
            Return localByteArray
        End Get

        Set(ByVal Value() As Byte)
            localByteArray = Value
        End Set

    End Property

    Public Property FilePath() As String

        'purpose: get and set FilePath prop.

        Get
            Return localFilePath
        End Get

        Set(ByVal Value As String)
            localFilePath = Value
        End Set

    End Property

    Sub New(ByVal filePath As String, ByVal byteArraySize As Integer)

        'purpose: constructor, allows setting of file path info.
        '         and byte array size info.

        'set local file path info
        localFilePath = filePath

        'dimension the size of the byte array
        ReDim localByteArray(byteArraySize)

    End Sub

End Class

